import { BatchWriteItemCommand, DynamoDBClient, } from "@aws-sdk/client-dynamodb";
import { readFileSync } from "fs";
const client = new DynamoDBClient({});
export async function handler(event) {
    switch (event.RequestType) {
        case "Create":
        case "Update":
            return insert();
        case "Delete":
            return remove();
    }
}
async function insert() {
    const file = readFileSync(`dynamodb-cache.json`, "utf8");
    const data = JSON.parse(file);
    // Chunk array into batches of 25
    const chunked = data.reduce((acc, curr, i) => {
        const index = Math.floor(i / 25);
        acc[index] = [...(acc[index] || []), curr];
        return acc;
    }, []);
    const TableName = process.env.CACHE_DYNAMO_TABLE;
    const promises = chunked.map((chunk) => {
        const params = {
            RequestItems: {
                [TableName]: chunk.map((item) => ({
                    PutRequest: {
                        Item: item,
                    },
                })),
            },
        };
        return client.send(new BatchWriteItemCommand(params));
    });
    await Promise.all(promises);
    return {
        PhysicalResourceId: "dynamodb-cache",
        Data: {},
    };
}
async function remove() {
    // Do we want to actually delete anything here?
    return {
        PhysicalResourceId: "dynamodb-cache",
        Data: {},
    };
}
