import { convertFrom } from "../event-mapper";
import { debug } from "../logger";
//#override imports
import { StreamingServerResponse } from "../http/responseStreaming";
import { processInternalEvent } from "./routing/default.js";
import { addOpenNextHeader, fixCacheHeaderForHtmlPages, fixISRHeaders, fixSWRCacheHeader, revalidateIfRequired, } from "./routing/util";
//#endOverride
//#override lambdaHandler
export const lambdaHandler = awslambda.streamifyResponse(async function (event, responseStream) {
    debug("event", event);
    // Handler warmer
    if ("type" in event) {
        // @ts-ignore formatWarmerResponse defined in lambdaHandler
        const result = await formatWarmerResponse(event);
        responseStream.end(Buffer.from(JSON.stringify(result)), "utf-8");
        return;
    }
    // Parse Lambda event and create Next.js request
    const internalEvent = convertFrom(event);
    // WORKAROUND: Set `x-forwarded-host` header (AWS specific) — https://github.com/serverless-stack/open-next#workaround-set-x-forwarded-host-header-aws-specific
    if (internalEvent.headers["x-forwarded-host"]) {
        internalEvent.headers.host = internalEvent.headers["x-forwarded-host"];
    }
    const createServerResponse = (method, headers) => {
        // sets the accept-encoding for responseStreaming.ts to set "content-encoding"
        headers["accept-encoding"] = internalEvent.headers["accept-encoding"];
        return new StreamingServerResponse({
            method,
            headers,
            responseStream,
            // We need to fix the cache header before sending any response
            fixHeaders: (headers) => {
                fixCacheHeaderForHtmlPages(internalEvent.rawPath, headers);
                fixSWRCacheHeader(headers);
                addOpenNextHeader(headers);
                fixISRHeaders(headers);
            },
            // This run in the callback of the response stream end
            onEnd: async (headers) => {
                await revalidateIfRequired(internalEvent.headers.host, internalEvent.rawPath, headers);
            },
        });
    };
    const preprocessResult = await processInternalEvent(internalEvent, createServerResponse);
    if ("type" in preprocessResult) {
        const headers = preprocessResult.headers;
        const res = createServerResponse("GET", headers);
        setImmediate(() => {
            res.writeHead(preprocessResult.statusCode, headers);
            res.write(preprocessResult.body);
            res.end();
        });
    }
    else {
        const { req, res, isExternalRewrite, internalEvent: overwrittenInternalEvent, } = preprocessResult;
        //@ts-expect-error - processRequest is already defined in serverHandler.ts
        await processRequest(req, res, overwrittenInternalEvent, isExternalRewrite);
    }
});
//#endOverride
