interface DangerousOptions {
    /**
     * The dynamo db cache is used for revalidateTags and revalidatePath.
     * @default false
     */
    disableDynamoDBCache?: boolean;
    /**
     * The incremental cache is used for ISR and SSG.
     * Disable this only if you use only SSR
     * @default false
     */
    disableIncrementalCache?: boolean;
}
interface BuildOptions {
    /**
     * Minify the server bundle.
     * @default false
     */
    minify?: boolean;
    /**
     * Print debug information.
     * @default false
     */
    debug?: boolean;
    /**
     * Enable streaming mode.
     * @default false
     */
    streaming?: boolean;
    /**
     * The command to build the Next.js app.
     * @default `npm run build`, `yarn build`, or `pnpm build` based on the lock file found in the app's directory or any of its parent directories.
     * @example
     * ```ts
     * build({
     *   buildCommand: "pnpm custom:build",
     * });
     * ```
     */
    /**
     * Dangerous options. This break some functionnality but can be useful in some cases.
     */
    dangerous?: DangerousOptions;
    buildCommand?: string;
    /**
     * The path to the target folder of build output from the `buildCommand` option (the path which will contain the `.next` and `.open-next` folders). This path is relative from the current process.cwd().
     * @default "."
     */
    buildOutputPath?: string;
    /**
     * The path to the root of the Next.js app's source code. This path is relative from the current process.cwd().
     * @default "."
     */
    appPath?: string;
}
export type PublicFiles = {
    files: string[];
};
export declare function build(opts?: BuildOptions): Promise<void>;
export {};
